@@ -2,6 +2,7 @@ import logging
 import os
 from decimal import Decimal
 from pathlib import Path
+from urllib.parse import urljoin
 
 import swapper
 from django.conf import settings
@@ -12,6 +13,7 @@ from django.utils.functional import cached_property
 from django.utils.module_loading import import_string
 from django.utils.translation import gettext_lazy as _
 from private_storage.fields import PrivateFileField
+from private_storage.storage.files import PrivateFileSystemStorage
 
 from openwisp_users.mixins import OrgMixin
 from openwisp_utils.base import TimeStampedEditableModel
@@ -28,6 +30,7 @@ from ..hardware import (
     FIRMWARE_IMAGE_TYPE_CHOICES,
     REVERSE_FIRMWARE_IMAGE_MAP,
 )
+from ..settings import FIRMWARE_API_BASEURL, IMAGE_URL_PATH
 from ..swapper import get_model_name, load_model
 from ..tasks import (
     batch_upgrade_operation,
@@ -102,13 +105,15 @@ class AbstractBuild(TimeStampedEditableModel):
 
     def clean(self):
         # Make sure that ('category__organization', 'os') is unique too
+        try:
+            category = self.category
+        except ObjectDoesNotExist:
+            return
         if not self.os:
             return
         if (
             load_model('Build')
-            .objects.filter(
-                category__organization=self.category.organization, os=self.os,
-            )
+            .objects.filter(category__organization=category.organization, os=self.os)
             .exclude(pk=self.pk)
             .exists()
         ):
@@ -116,7 +121,7 @@ class AbstractBuild(TimeStampedEditableModel):
                 {
                     'os': _(
                         f'A build with this OS identifier ("{self.os}") and '
-                        f'organization ("{self.category.organization}") already exists'
+                        f'organization ("{category.organization}") already exists'
                     )
                 }
             )
@@ -172,7 +177,12 @@ def get_build_directory(instance, filename):
 class AbstractFirmwareImage(TimeStampedEditableModel):
     build = models.ForeignKey(get_model_name('Build'), on_delete=models.CASCADE)
     file = PrivateFileField(
-        'File', upload_to=get_build_directory, max_file_size=app_settings.MAX_FILE_SIZE
+        'File',
+        upload_to=get_build_directory,
+        max_file_size=app_settings.MAX_FILE_SIZE,
+        storage=PrivateFileSystemStorage(
+            base_url=urljoin(FIRMWARE_API_BASEURL, IMAGE_URL_PATH)
+        ),
     )
     type = models.CharField(
         blank=True,
@@ -257,6 +267,8 @@ class AbstractDeviceFirmware(TimeStampedEditableModel):
         abstract = True
 
     def clean(self):
+        if not hasattr(self, 'image') or not hasattr(self, 'device'):
+            return
         if self.image.build.category.organization != self.device.organization:
             raise ValidationError(
                 {
@@ -490,7 +502,9 @@ class AbstractUpgradeOperation(TimeStampedEditableModel):
     device = models.ForeignKey(
         swapper.get_model_name('config', 'Device'), on_delete=models.CASCADE
     )
-    image = models.ForeignKey(get_model_name('FirmwareImage'), on_delete=models.CASCADE)
+    image = models.ForeignKey(
+        get_model_name('FirmwareImage'), null=True, on_delete=models.SET_NULL
+    )
     status = models.CharField(
         max_length=12, choices=STATUS_CHOICES, default=STATUS_CHOICES[0][0]
     )
