@@ -5,7 +5,8 @@ from openwisp_controller.connection import settings as conn_settings
 CUSTOM_OPENWRT_IMAGES = getattr(settings, 'OPENWISP_CUSTOM_OPENWRT_IMAGES', None)
 # fmt: off
 UPGRADERS_MAP = getattr(settings, 'OPENWISP_FIRMWARE_UPGRADERS_MAP', {
-    conn_settings.DEFAULT_UPDATE_STRATEGIES[0][0]: 'openwisp_firmware_upgrader.upgraders.openwrt.OpenWrt'
+    conn_settings.DEFAULT_UPDATE_STRATEGIES[0][0]: 'openwisp_firmware_upgrader.upgraders.openwrt.OpenWrt',
+    conn_settings.DEFAULT_UPDATE_STRATEGIES[1][0]: 'openwisp_firmware_upgrader.upgraders.openwisp.OpenWisp1'
 })
 # fmt: on
 
@@ -19,7 +20,11 @@ RETRY_OPTIONS = getattr(
     dict(max_retries=4, retry_backoff=60, retry_backoff_max=600, retry_jitter=True),
 )
 
-TASK_TIMEOUT = getattr(settings, 'OPENWISP_FIRMWARE_UPGRADER_TASK_TIMEOUT', 600)
+TASK_TIMEOUT = getattr(settings, 'OPENWISP_FIRMWARE_UPGRADER_TASK_TIMEOUT', 1500)
 
-FIRMWARE_UPGRADER_API = getattr(settings, 'OPENWISP_FIRMWARE_UPGRADER_API', False)
+FIRMWARE_UPGRADER_API = getattr(settings, 'OPENWISP_FIRMWARE_UPGRADER_API', True)
+FIRMWARE_API_BASEURL = getattr(settings, 'OPENWISP_FIRMWARE_API_BASEURL', '/')
 OPENWRT_SETTINGS = getattr(settings, 'OPENWISP_FIRMWARE_UPGRADER_OPENWRT_SETTINGS', {})
+
+# Path of urls that need to be refered in migrations files.
+IMAGE_URL_PATH = 'firmware/'
