@@ -14,6 +14,7 @@ from openwisp_firmware_upgrader.api.serializers import (
 )
 from openwisp_firmware_upgrader.tests.base import TestUpgraderMixin
 from openwisp_users.tests.utils import TestMultitenantAdminMixin
+from openwisp_utils.tests import AssertNumQueriesSubTestMixin
 
 from ..swapper import load_model
 
@@ -28,25 +29,23 @@ OrganizationUser = swapper.load_model('openwisp_users', 'OrganizationUser')
 user_model = get_user_model()
 
 
-class TestAPIUpgraderMixin(TestMultitenantAdminMixin, TestUpgraderMixin):
+class TestAPIUpgraderMixin(
+    AssertNumQueriesSubTestMixin, TestMultitenantAdminMixin, TestUpgraderMixin
+):
     def setUp(self):
         self.org = self._get_org()
         self.operator = self._create_operator(organizations=[self.org])
+        self.administrator = self._create_administrator(organizations=[self.org])
         self._login()
 
-    def _make_operator_org_manager(self):
-        orgrelation = OrganizationUser.objects.get(user=self.operator)
-        orgrelation.is_admin = True
-        orgrelation.save()
-
-    def _obtain_auth_token(self, username='operator', password='tester'):
+    def _obtain_auth_token(self, username='administrator', password='tester'):
         params = {'username': username, 'password': password}
         url = reverse('users:user_auth_token')
         r = self.client.post(url, params)
         self.assertEqual(r.status_code, 200)
         return r.data['token']
 
-    def _login(self, username='operator', password='tester'):
+    def _login(self, username='administrator', password='tester'):
         token = self._obtain_auth_token(username, password)
         self.client = Client(HTTP_AUTHORIZATION='Bearer ' + token)
 
@@ -82,7 +81,7 @@ class TestBuildViews(TestAPIUpgraderMixin, TestCase):
             for build in Build.objects.all().order_by('-created')
         ]
         url = reverse('upgrader:api_build_list')
-        with self.assertNumQueries(3):
+        with self.assertNumQueries(5):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
 
@@ -92,17 +91,45 @@ class TestBuildViews(TestAPIUpgraderMixin, TestCase):
 
         build1 = self._create_build(category=category1)
         build2 = self._create_build(version='0.2', category=category2)
+        build3 = self._create_build(version='0.2.1', category=category2)
         url = reverse('upgrader:api_build_list')
 
         filter_params = dict(category=category1.pk)
-        with self.assertNumQueries(4):
+        with self.assertNumQueries(6):
             r = self.client.get(url, filter_params)
         self.assertEqual(r.data['results'], [self._serialize_build(build1)])
 
         filter_params = dict(category=category2.pk)
-        with self.assertNumQueries(4):
+        with self.assertNumQueries(6):
             r = self.client.get(url, filter_params)
-        self.assertEqual(r.data['results'], [self._serialize_build(build2)])
+        self.assertEqual(
+            r.data['results'],
+            [self._serialize_build(build3), self._serialize_build(build2)],
+        )
+
+        with self.subTest('test version filter'):
+            with self.assertNumQueries(5):
+                r = self.client.get(url, {'version': '0.2'})
+            self.assertEqual(r.data['results'], [self._serialize_build(build2)])
+
+            with self.assertNumQueries(5):
+                r = self.client.get(url, {'version': '0.2.1'})
+            self.assertEqual(r.data['results'], [self._serialize_build(build3)])
+
+        with self.subTest('test os filter'):
+            build1.os = 'abcdefg'
+            build1.save(update_fields=('os',))
+            build2.os = 'abcdefg-old'
+            build2.save(update_fields=('os',))
+            with self.assertNumQueries(5):
+                r = self.client.get(url, {'os': build1.os})
+            self.assertEqual(r.data['results'], [self._serialize_build(build1)])
+
+        with self.subTest('test version, os, category should AND'):
+            filter_params.update({'version': '0.2', 'os': 'abcdefg-old'})
+            with self.assertNumQueries(6):
+                r = self.client.get(url, filter_params)
+            self.assertEqual(r.data['results'], [self._serialize_build(build2)])
 
     def test_build_list_filter_org(self):
         org2 = self._create_org(name='New org', slug='new-org')
@@ -120,7 +147,7 @@ class TestBuildViews(TestAPIUpgraderMixin, TestCase):
         serialized_list = [
             self._serialize_build(build),
         ]
-        with self.assertNumQueries(3):
+        with self.assertNumQueries(5):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
 
@@ -128,7 +155,7 @@ class TestBuildViews(TestAPIUpgraderMixin, TestCase):
         serialized_list = [
             self._serialize_build(build2),
         ]
-        with self.assertNumQueries(3):
+        with self.assertNumQueries(5):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
 
@@ -186,7 +213,7 @@ class TestBuildViews(TestAPIUpgraderMixin, TestCase):
         build = self._create_build()
         serialized = self._serialize_build(build)
         url = reverse('upgrader:api_build_detail', args=[build.pk])
-        with self.assertNumQueries(2):
+        with self.assertNumQueries(5):
             r = self.client.get(url)
         self.assertEqual(r.data, serialized)
 
@@ -199,7 +226,7 @@ class TestBuildViews(TestAPIUpgraderMixin, TestCase):
             'version': '20.04',
             'changelog': 'PUT update',
         }
-        with self.assertNumQueries(9):
+        with self.assertNumQueries(10):
             r = self.client.put(url, data, content_type='application/json')
         self.assertEqual(r.data['id'], str(build.pk))
         self.assertEqual(r.data['category'], build.category.pk)
@@ -210,7 +237,7 @@ class TestBuildViews(TestAPIUpgraderMixin, TestCase):
         build = self._create_build()
         url = reverse('upgrader:api_build_detail', args=[build.pk])
         data = dict(changelog='PATCH update')
-        with self.assertNumQueries(8):
+        with self.assertNumQueries(9):
             r = self.client.patch(url, data, content_type='application/json')
         self.assertEqual(r.data['id'], str(build.pk))
         self.assertEqual(r.data['category'], build.category.pk)
@@ -231,7 +258,7 @@ class TestBuildViews(TestAPIUpgraderMixin, TestCase):
         self.assertEqual(DeviceFirmware.objects.count(), 0)
         with self.subTest('Existing build'):
             url = reverse('upgrader:api_build_batch_upgrade', args=[build.pk])
-            with self.assertNumQueries(7):
+            with self.assertNumQueries(8):
                 r = self.client.post(url)
             self.assertEqual(BatchUpgradeOperation.objects.count(), 1)
             self.assertEqual(DeviceFirmware.objects.count(), 0)
@@ -251,7 +278,7 @@ class TestBuildViews(TestAPIUpgraderMixin, TestCase):
         self.assertEqual(BatchUpgradeOperation.objects.count(), 0)
 
         url = reverse('upgrader:api_build_batch_upgrade', args=[env['build2'].pk])
-        with self.assertNumQueries(7):
+        with self.assertNumQueries(10):
             r = self.client.get(url)
         self.assertEqual(r.status_code, 200)
         device_fw_list = [
@@ -263,7 +290,7 @@ class TestBuildViews(TestAPIUpgraderMixin, TestCase):
 
     def test_build_upgradeable_404(self):
         url = reverse('upgrader:api_build_batch_upgrade', args=[uuid.uuid4()])
-        with self.assertNumQueries(2):
+        with self.assertNumQueries(4):
             r = self.client.get(url)
         self.assertEqual(r.status_code, 404)
         self.assertEqual(r.json(), {'detail': 'Not found.'})
@@ -281,10 +308,11 @@ class TestCategoryViews(TestAPIUpgraderMixin, TestCase):
         org2 = self._create_org(name='org2', slug='org2')
         self.tearDown()
         self.operator.openwisp_users_organization.all().delete()
-        OrganizationUser.objects.create(user=self.operator, organization=org2)
-
+        OrganizationUser.objects.create(
+            user=self.administrator, organization=org2, is_admin=True
+        )
         url = reverse('upgrader:api_category_detail', args=[category.pk])
-        with self.assertNumQueries(2):
+        with self.assertNumQueries(4):
             r = self.client.get(url)
         self.assertEqual(r.status_code, 404)
 
@@ -306,14 +334,16 @@ class TestCategoryViews(TestAPIUpgraderMixin, TestCase):
             for category in Category.objects.all().order_by('-name')
         ]
         url = reverse('upgrader:api_category_list')
-        with self.assertNumQueries(3):
+        with self.assertNumQueries(5):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
 
     def test_category_list_filter_org(self):
         org2 = self._create_org(name='New org', slug='new-org')
-        self._create_operator(
-            organizations=[org2], username='operator2', email='operator2@test.com'
+        self._create_administrator(
+            organizations=[org2],
+            username='administrator2',
+            email='administrator2@test.com',
         )
 
         category = self._create_category()
@@ -321,19 +351,19 @@ class TestCategoryViews(TestAPIUpgraderMixin, TestCase):
 
         url = reverse('upgrader:api_category_list')
 
-        self._login('operator', 'tester')
+        self._login('administrator', 'tester')
         serialized_list = [
             self._serialize_category(category),
         ]
-        with self.assertNumQueries(3):
+        with self.assertNumQueries(5):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
 
-        self._login('operator2', 'tester')
+        self._login('administrator2', 'tester')
         serialized_list = [
             self._serialize_category(category2),
         ]
-        with self.assertNumQueries(3):
+        with self.assertNumQueries(5):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
 
@@ -382,7 +412,7 @@ class TestCategoryViews(TestAPIUpgraderMixin, TestCase):
         category = self._get_category()
         serialized = self._serialize_category(category)
         url = reverse('upgrader:api_category_detail', args=[category.pk])
-        with self.assertNumQueries(2):
+        with self.assertNumQueries(5):
             r = self.client.get(url)
         self.assertEqual(r.data, serialized)
 
@@ -393,7 +423,7 @@ class TestCategoryViews(TestAPIUpgraderMixin, TestCase):
             'name': 'New name',
             'organization': category.organization.pk,
         }
-        with self.assertNumQueries(9):
+        with self.assertNumQueries(10):
             r = self.client.put(url, data, content_type='application/json')
         self.assertEqual(r.data['id'], str(category.pk))
         self.assertEqual(r.data['name'], 'New name')
@@ -432,13 +462,15 @@ class TestBatchUpgradeOperationViews(TestAPIUpgraderMixin, TestCase):
 
         org2 = self._create_org(name='org2', slug='org2')
         self.tearDown()
-        self.operator.openwisp_users_organization.all().delete()
-        OrganizationUser.objects.create(user=self.operator, organization=org2)
+        self.administrator.openwisp_users_organization.all().delete()
+        OrganizationUser.objects.create(
+            user=self.administrator, organization=org2, is_admin=True
+        )
 
         url = reverse(
             'upgrader:api_batchupgradeoperation_detail', args=[env['build2'].pk]
         )
-        with self.assertNumQueries(2):
+        with self.assertNumQueries(4):
             r = self.client.get(url)
         self.assertEqual(r.status_code, 404)
 
@@ -460,7 +492,7 @@ class TestBatchUpgradeOperationViews(TestAPIUpgraderMixin, TestCase):
         operation = BatchUpgradeOperation.objects.get(build=env['build2'])
         serialized_list = [self._serialize_upgrade_env(operation)]
         url = reverse('upgrader:api_batchupgradeoperation_list')
-        with self.assertNumQueries(3):
+        with self.assertNumQueries(5):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
 
@@ -475,21 +507,21 @@ class TestBatchUpgradeOperationViews(TestAPIUpgraderMixin, TestCase):
             self._serialize_upgrade_env(operation)
             for operation in BatchUpgradeOperation.objects.order_by('-created')
         ]
-        with self.assertNumQueries(3):
+        with self.assertNumQueries(5):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
 
         operation = BatchUpgradeOperation.objects.get(build=env['build1'])
         serialized_list = [self._serialize_upgrade_env(operation)]
         filter_params = dict(build=env['build1'].pk)
-        with self.assertNumQueries(4):
+        with self.assertNumQueries(6):
             r = self.client.get(url, filter_params)
         self.assertEqual(r.data['results'], serialized_list)
 
         operation = BatchUpgradeOperation.objects.get(build=env['build2'])
         serialized_list = [self._serialize_upgrade_env(operation)]
         filter_params = dict(build=env['build2'].pk)
-        with self.assertNumQueries(4):
+        with self.assertNumQueries(6):
             r = self.client.get(url, filter_params)
         self.assertEqual(r.data['results'], serialized_list)
 
@@ -500,7 +532,7 @@ class TestBatchUpgradeOperationViews(TestAPIUpgraderMixin, TestCase):
             ).order_by('-created')
         ]
         filter_params = dict(status='in-progress')
-        with self.assertNumQueries(2):
+        with self.assertNumQueries(4):
             r = self.client.get(url, filter_params)
         self.assertEqual(r.data['results'], serialized_list)
 
@@ -511,7 +543,7 @@ class TestBatchUpgradeOperationViews(TestAPIUpgraderMixin, TestCase):
             ).order_by('-created')
         ]
         filter_params = dict(status='success')
-        with self.assertNumQueries(2):
+        with self.assertNumQueries(4):
             r = self.client.get(url, filter_params)
         self.assertEqual(r.data['results'], serialized_list)
 
@@ -532,14 +564,14 @@ class TestBatchUpgradeOperationViews(TestAPIUpgraderMixin, TestCase):
         self._login('operator', 'tester')
         operation = BatchUpgradeOperation.objects.get(build=env['build2'])
         serialized_list = [self._serialize_upgrade_env(operation)]
-        with self.assertNumQueries(3):
+        with self.assertNumQueries(5):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
 
         self._login('operator2', 'tester')
         operation2 = BatchUpgradeOperation.objects.get(build=env2['build2'])
         serialized_list = [self._serialize_upgrade_env(operation2)]
-        with self.assertNumQueries(3):
+        with self.assertNumQueries(5):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
 
@@ -582,7 +614,7 @@ class TestBatchUpgradeOperationViews(TestAPIUpgraderMixin, TestCase):
         operation = BatchUpgradeOperation.objects.get(build=env['build2'])
         serialized = self._serialize_upgrade_env(operation, action='detail')
         url = reverse('upgrader:api_batchupgradeoperation_detail', args=[operation.pk])
-        with self.assertNumQueries(4):
+        with self.assertNumQueries(7):
             r = self.client.get(url)
         self.assertEqual(r.data, serialized)
 
@@ -627,7 +659,7 @@ class TestFirmwareImageViews(TestAPIUpgraderMixin, TestCase):
             for image in FirmwareImage.objects.all().order_by('-created')
         ]
         url = reverse('upgrader:api_firmware_list', args=[image.build.pk])
-        with self.assertNumQueries(4):
+        with self.assertNumQueries(6):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
 
@@ -645,14 +677,14 @@ class TestFirmwareImageViews(TestAPIUpgraderMixin, TestCase):
         url = reverse('upgrader:api_firmware_list', args=[image.build.pk])
 
         filter_params = dict(type=self.TPLINK_4300_IMAGE)
-        with self.assertNumQueries(4):
+        with self.assertNumQueries(6):
             r = self.client.get(url, filter_params)
         self.assertEqual(r.data['results'], [self._serialize_image(image)])
 
         url = reverse('upgrader:api_firmware_list', args=[image.build.pk])
 
         filter_params = dict(type=self.TPLINK_4300_IL_IMAGE)
-        with self.assertNumQueries(4):
+        with self.assertNumQueries(6):
             r = self.client.get(url, filter_params)
         self.assertEqual(r.data['results'], [self._serialize_image(image2)])
 
@@ -671,14 +703,14 @@ class TestFirmwareImageViews(TestAPIUpgraderMixin, TestCase):
 
         self._login('operator', 'tester')
         serialized_list = [self._serialize_image(image)]
-        with self.assertNumQueries(4):
+        with self.assertNumQueries(6):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
 
         url = reverse('upgrader:api_firmware_list', args=[image2.build.pk])
         self._login('operator2', 'tester')
         serialized_list = [self._serialize_image(image2)]
-        with self.assertNumQueries(4):
+        with self.assertNumQueries(6):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
 
@@ -745,7 +777,7 @@ class TestFirmwareImageViews(TestAPIUpgraderMixin, TestCase):
         image = self._create_firmware_image()
         serialized = self._serialize_image(image)
         url = reverse('upgrader:api_firmware_detail', args=[image.build.pk, image.pk])
-        with self.assertNumQueries(3):
+        with self.assertNumQueries(8):
             r = self.client.get(url)
         self.assertEqual(r.data, serialized)
 
@@ -753,7 +785,7 @@ class TestFirmwareImageViews(TestAPIUpgraderMixin, TestCase):
         image = self._create_firmware_image()
         self.assertEqual(FirmwareImage.objects.count(), 1)
         url = reverse('upgrader:api_firmware_detail', args=[image.build.pk, image.pk])
-        with self.assertNumQueries(10):
+        with self.assertNumQueries(11):
             r = self.client.delete(url)
         self.assertEqual(r.status_code, 204)
         self.assertEqual(FirmwareImage.objects.count(), 0)
@@ -764,8 +796,8 @@ class TestFirmwareImageViews(TestAPIUpgraderMixin, TestCase):
             content = f.read()
         url = reverse('upgrader:api_firmware_download', args=[image.build.pk, image.pk])
         with self.subTest("Test as operator"):
-            self._make_operator_org_manager()
-            with self.assertNumQueries(6):
+            self._login('operator', 'tester')
+            with self.assertNumQueries(8):
                 response = self.client.get(url)
             self.assertEqual(response.getvalue(), content)
         with self.subTest("Test as superuser"):
@@ -816,7 +848,24 @@ class TestOrgAPIMixin(TestAPIUpgraderMixin, TestCase):
             self._serialize_build(build)
             for build in Build.objects.all().order_by('-created')
         ]
-        with self.assertNumQueries(3):
+        with self.assertNumQueries(5):
             r = self.client.get(url)
         self.assertEqual(r.data['results'], serialized_list)
         self.assertEqual(r.status_code, 200)
+
+
+class TestApiMisc(TestAPIUpgraderMixin, TestCase):
+    def test_api_docs(self):
+        url = reverse('schema-swagger-ui')
+
+        with self.subTest('not authenticated'):
+            response = self.client.get(url)
+            self.assertEqual(response.status_code, 403)
+
+        with self.subTest('authenticated'):
+            self._create_operator(
+                username='admin', email='admin@test.com', is_superuser=True
+            )
+            self._login('admin', 'tester')
+            response = self.client.get(url)
+            self.assertEqual(response.status_code, 403)
