@@ -7,7 +7,7 @@ from celery.exceptions import Retry
 from django.core.exceptions import ValidationError
 from django.test import TestCase, TransactionTestCase
 
-from openwisp_controller.config.models import Device
+from openwisp_utils.tests import capture_any_output
 
 from .. import settings as app_settings
 from ..hardware import FIRMWARE_IMAGE_MAP, REVERSE_FIRMWARE_IMAGE_MAP
@@ -24,6 +24,7 @@ FirmwareImage = load_model('FirmwareImage')
 UpgradeOperation = load_model('UpgradeOperation')
 DeviceConnection = swapper.load_model('connection', 'DeviceConnection')
 Credentials = swapper.load_model('connection', 'Credentials')
+Device = swapper.load_model('config', 'Device')
 
 
 class TestModels(TestUpgraderMixin, TestCase):
@@ -65,6 +66,18 @@ class TestModels(TestUpgraderMixin, TestCase):
         with self.subTest('validating the same object again should work'):
             b1.full_clean()
 
+        with self.subTest('validation error should be raised on empty category'):
+            try:
+                b2 = self._create_build(
+                    os=self.os + '_2', version='0.2', organization=org
+                )
+                b2.category = None
+                b2.full_clean()
+            except ValidationError as e:
+                self.assertIn('category', e.message_dict)
+            else:
+                self.fail('ValidationError not raised when build category is empty')
+
     def test_fw_str(self):
         fw = self._create_firmware_image()
         self.assertIn(str(fw.build), str(fw))
@@ -151,9 +164,7 @@ class TestModels(TestUpgraderMixin, TestCase):
             self.fail('ValidationError not raised')
 
     def test_invalid_board(self):
-        image = FIRMWARE_IMAGE_MAP[
-            'ar71xx-generic-tl-wdr4300-v1-squashfs-sysupgrade.bin'
-        ]
+        image = FIRMWARE_IMAGE_MAP[self.TPLINK_4300_IMAGE]
         boards = image['boards']
         del image['boards']
         err = None
@@ -242,6 +253,7 @@ class TestModels(TestUpgraderMixin, TestCase):
                 codename = '{}_{}'.format(action, model_name)
                 self.assertIn(codename, admin_permissions)
 
+    @capture_any_output()
     def test_create_for_device_validation_error(self):
         device_fw = self._create_device_firmware()
         device_fw.image.build.os = device_fw.device.os
@@ -249,6 +261,14 @@ class TestModels(TestUpgraderMixin, TestCase):
         result = DeviceFirmware.create_for_device(device_fw.device)
         self.assertIsNone(result)
 
+    def test_upgrade_operation_retention_on_image_delete(self):
+        device_fw = self._create_device_firmware()
+        uo = UpgradeOperation.objects.create(
+            device=device_fw.device, image=device_fw.image
+        )
+        FirmwareImage.objects.get(pk=device_fw.image.pk).delete()
+        self.assertEqual(UpgradeOperation.objects.get(pk=uo.pk).image, None)
+
 
 class TestModelsTransaction(TestUpgraderMixin, TransactionTestCase):
     _mock_updrade = 'openwisp_firmware_upgrader.upgraders.openwrt.OpenWrt.upgrade'
